package com.scipanama.photoUploader.platform;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.BitmapFactory;
import android.os.Environment;
import android.util.Log;
import android.widget.Toast;

import com.google.common.base.Preconditions;
import com.google.inject.Inject;
import com.scipanama.photoUploader.R;

public class FileInterface {
	
	private static final String TAG = "FileInterface";
	private static final String TAG_PREAMBLE = TAG.concat(" >===< ");
	
	@Inject private Context pContext;
	
	public File getRootDirectory(final FileType pType){
		
		Preconditions.checkNotNull(pType, "FileType is null!!");
		return pContext.getExternalFilesDir(pType.getDirectory());
	}
	
	public File getReferenceDirectory(final FileType pType, final String pReference){
		return new File(getRootDirectory(pType), pReference);
	}
	
	/**
	 * Creates a new picture file to send through the intent
	 * 
	 * @param pReference
	 * @return
	 */
	public File createPictureFile(final String pReference){
		//Is the SD card mounted???
		if (!Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)){
			Log.e(TAG, TAG_PREAMBLE.concat("No SD Card!!"));
			Toast.makeText(pContext, R.string.activity_gallery_error_storageNotMounted, Toast.LENGTH_LONG).show();
			return null;
		}
		
		//Check Folder		
		File mediaStorageDir = this.getReferenceDirectory(FileType.STORAGE, pReference);
		if (!mediaStorageDir.exists()){
			if (!mediaStorageDir.mkdirs()){
				Log.d(TAG, TAG_PREAMBLE.concat("failed to create directory"));
				return null;
			}
		}
		
		//Create File
		final String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss", Locale.US).format(new Date());
		return new File(mediaStorageDir.getPath(), timeStamp.concat(".jpg"));
	}
	
	/**
	 * Get a picture to diplay on the UI. Handles cach automatically.
	 * 
	 * @param pFile
	 * @return
	 */
	public Bitmap getBitmapFromCache(final File pFile){
		
		//File to be checked
		final File cachedPhoto = new File(pContext.getCacheDir(), pFile.getName());
		
		//Is there already a cachd file...
		if (cachedPhoto.exists()){
			
			//Return cachd file
			return BitmapFactory.decodeFile(cachedPhoto.getPath());
		}
		
		//...Or do we have to create a new one???
		else{
			final BitmapFactory.Options opts = new BitmapFactory.Options();
			opts.inSampleSize = 8;
			
			//Create Cach
			final Bitmap bmp = BitmapFactory.decodeFile(pFile.getPath(), opts);
			//final Bitmap result = Bitmap.createScaledBitmap(bmp, 150, 150, false);
			try {
				
				//Create cach file
				final FileOutputStream fos2 = new FileOutputStream(cachedPhoto);
				bmp.compress(CompressFormat.JPEG, 90, fos2);
				
			} catch (FileNotFoundException e) {
				Log.e(TAG, TAG_PREAMBLE.concat("Failure creating cache for ").concat(pFile.getName()), e);
			}
			
			//Return decoded bitmap
			return bmp;
			
		}		
		
	}
	
	/**
	 * Puts the reference in the Upload Queve
	 * 
	 * @param pReference
	 * @return has is been successfully inserted in the queve?
	 */
	public boolean markReferenceForUploading(final String pReference){
		
		final File newFolder = getReferenceDirectory(FileType.UPLOADING, pReference);
		final File oldFolder = getReferenceDirectory(FileType.STORAGE, pReference);
		
		if(!oldFolder.exists()){
			return false;
		}
		
		if (!newFolder.exists()){
			if(!newFolder.mkdirs()){
				return false;
			}
		}
		
		for (File photo: oldFolder.listFiles()){
			final File newPhoto = new File(newFolder.getPath(), photo.getName());
			if (!photo.renameTo(newPhoto)){
				return false;
			}
		}
		
		oldFolder.delete();
		
		//Ask ContentResolver to Sync
		pContext.getContentResolver().notifyChange(UriFunction.QUEVE.getUri(), null, true);
		return true;
	}
	
	public enum FileType{
		STORAGE(Environment.DIRECTORY_PICTURES),
		UPLOADING(Environment.DIRECTORY_DOWNLOADS);
		
		private final String mDirectory;
		
		private FileType(final String pDirectory){
			this.mDirectory = pDirectory;
		}
		
		public String getDirectory(){
			return mDirectory;			
		}
	}
	

}
